#region Copyright (C) 2003-2010 Stimulsoft
/*
{*******************************************************************}
{																	}
{	Stimulsoft Reports       										}
{																	}
{	Copyright (C) 2003-2010 Stimulsoft     							}
{	ALL RIGHTS RESERVED												}
{																	}
{	The entire contents of this file is protected by U.S. and		}
{	International Copyright Laws. Unauthorized reproduction,		}
{	reverse-engineering, and distribution of all or any portion of	}
{	the code contained in this file is strictly prohibited and may	}
{	result in severe civil and criminal penalties and will be		}
{	prosecuted to the maximum extent possible under the law.		}
{																	}
{	RESTRICTIONS													}
{																	}
{	THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES			}
{	ARE CONFIDENTIAL AND PROPRIETARY								}
{	TRADE SECRETS OF Stimulsoft										}
{																	}
{	CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON		}
{	ADDITIONAL RESTRICTIONS.										}
{																	}
{*******************************************************************}
*/
#endregion Copyright (C) 2003-2010 Stimulsoft

using System;
using System.ComponentModel;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Diagnostics;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Reflection;
using System.Reflection.Emit;
using Stimulsoft.Base;
using Stimulsoft.Base.Drawing;
using Stimulsoft.Base.Localization;
using Stimulsoft.Base.Services;
using System.Data.SQLite;

namespace Stimulsoft.Database
{
    [StiServiceBitmap(typeof(Stimulsoft.Report.Dictionary.StiDataAdapterService), "Stimulsoft.Report.Bmp.DataAdapter.bmp")]
    [StiServiceCategoryBitmap(typeof(Stimulsoft.Report.Dictionary.StiDataAdapterService), "Stimulsoft.Report.Bmp.DataAdapter.bmp")]		
    public class StiSQLiteMetadataProvider : BaseMetadataProvider
    {
        #region StiService override
        /// <summary>
        /// Gets a service name.
        /// </summary>
        public override string ServiceName
        {
            get
            {
                return "SQLite Metadata Provider";
            }
        }
        #endregion

        #region Properties
        [Browsable(true)]
        public SQLiteConnection Connection
        { 
            get 
            { 
                return internalConnection as SQLiteConnection; 
            } 
            set 
            {
                if (internalConnection != value)
                {
                    internalConnection = value;
                }
            }
        }        

        public override bool IsConnected
        {
            get
            {
                if (Connection != null)
                {
                    return (Connection.State & ConnectionState.Open) == ConnectionState.Open;
                }
                else return false;
            }
            set
            {
                base.IsConnected = value;
            }
        }

        public override bool IsCanCreateInternalConnection
        {
            get
            {
                return true;
            }
        }
        #endregion

        private SQLQualifiedName fDefaultDatabaseName = null;

        #region Methods
        public override void ViewQuery(
            IDbConnection connection, 
            QueryBuilder queryBuilder, DataGridView dataGridView)
        {
            using (SQLiteCommand cmd = connection.CreateCommand() as SQLiteCommand)
            {
                cmd.CommandTimeout = Stimulsoft.Report.StiOptions.Dictionary.QueryBuilderConnectTimeout;
                cmd.CommandText = queryBuilder.SQL;

                // handle the query parameters
                if (queryBuilder.Parameters.Count > 0)
                {
                    for (int i = 0; i < queryBuilder.Parameters.Count; i++)
                    {
                        SQLiteParameter p = new SQLiteParameter();
                        p.ParameterName = queryBuilder.Parameters[i].FullName;
                        p.DbType = queryBuilder.Parameters[i].DataType;
                        cmd.Parameters.Add(p);
                    }

                    using (QueryParametersForm form = new QueryParametersForm(queryBuilder.Parameters, cmd))
                    {
                        form.ShowDialog();
                    }
                }

                using (SQLiteDataAdapter adapter = new SQLiteDataAdapter(cmd))
                {
                    DataSet dataset = new DataSet();

                    try
                    {
                        adapter.Fill(dataset, "QueryResult");
                        dataGridView.DataSource = dataset.Tables["QueryResult"];
                    }
                    catch (Exception ex)
                    {
                        Stimulsoft.Base.StiExceptionProvider.Show(ex);
                    }
                }
            }
        }

        protected override void DoConnect()
        {
            base.DoConnect();

            CheckConnectionSet();

            try
            {
                Connection.Open();
            }
            catch
            {
                throw;
            }
        }

        protected override void DoDisconnect()
        {
            base.DoDisconnect();

            CheckConnectionSet();
            Connection.Close();
        }

        protected override void CheckConnectionSet()
        {
            if (Connection == null)
            {
                throw new Exception(String.Format(StiLocalization.Get("QueryBuilder", "NoConnectionObject"), "Connection"));
            }
        }

        protected override IDataReader PrepareSQLDatasetInternal(string sql, bool schemaOnly)
        {
            SQLiteCommand command = null;
            SQLiteDataReader reader = null;

            if (!IsConnected) Connect();

            command = Connection.CreateCommand();
            command.CommandText = sql;

            if (schemaOnly)
            {
                reader = command.ExecuteReader(CommandBehavior.SchemaOnly);
            }
            else
            {
                reader = command.ExecuteReader();
            }

            return reader;
        }

        protected override void ExecSQLInternal(string sql)
        {
            if (!IsConnected) Connect();

            SQLiteCommand command = Connection.CreateCommand();
            command.CommandText = sql;
            command.ExecuteNonQuery();
        }

        public override SQLQualifiedName GetDefaultDatabaseName(BaseSyntaxProvider ASyntaxProvider)
        {
            IDataReader reader = null;

            if (fDefaultDatabaseName != null)
            {
                return fDefaultDatabaseName;
            }
            else
            {
                fDefaultDatabaseName = new SQLQualifiedName(ASyntaxProvider);

                try
                {
                    reader = ExecSQL("select DATABASE()", false);

                    if (reader.Read() && reader.FieldCount > 0 && !reader.IsDBNull(0))
                    {
                        fDefaultDatabaseName.AddName(reader.GetString(0), false);
                    }
                }
                catch // eat exceptions
                {
                    fDefaultDatabaseName = null;
                }
                finally
                {
                    if (reader != null && !reader.IsClosed)
                    {
                        reader.Close();
                    }
                }
            }

            return fDefaultDatabaseName;
        }

        public override void LoadMetadataObjects(
            BaseSyntaxProvider ASyntaxProvider, 
            MetadataFilter AMetadataFilter, 
            MetadataContainer AMetadataContainer, 
            SQLQualifiedName ADatabase)
        {
            DataTable schemaTable;
            MetadataObjectFetcherFromDatatable mof;

            if (!IsConnected) Connect();

            try
            {
                // load tables
                schemaTable = Connection.GetSchema("TABLES", null);
                mof = new MetadataObjectFetcherFromDatatable(ASyntaxProvider, this);
                mof.SchemaFieldName = "TABLE_SCHEMA";
                mof.NameFieldName = "TABLE_NAME";
                mof.TypeFieldName = "TABLE_TYPE";
                mof.TableType = new string[] { "BASE TABLE" };
                mof.SystemTableType = new string[] { "SYSTEM TABLE" };
                mof.Datatable = schemaTable;
                mof.LoadMetadata(AMetadataFilter, AMetadataContainer, ADatabase);

                // load views
                schemaTable = Connection.GetSchema("VIEWS", null);

                mof = new MetadataObjectFetcherFromDatatable(ASyntaxProvider, this);
                mof.SchemaFieldName = "TABLE_SCHEMA";
                mof.NameFieldName = "TABLE_NAME";
                mof.Datatable = schemaTable;
                mof.DefaultObjectClass = typeof(MetadataView);

                mof.LoadMetadata(AMetadataFilter, AMetadataContainer, ADatabase);
            }
            catch (SQLiteException e)
            {
                throw (e);
            }
            catch (Exception e)
            {
                throw (e);
            }

        }

        public override void LoadMetadataRelations(
            BaseSyntaxProvider ASyntaxProvider, 
            MetadataFilter AMetadataFilter, 
            MetadataContainer AMetadataContainer, 
            SQLQualifiedName ADatabase)
        {
            DataTable schemaTable;
            MetadataRelationsFetcherFromDatatable mrf;

            if (!IsConnected) Connect();

            // load FKs
            try
            {
                schemaTable = Connection.GetSchema("ForeignKeys", null);

                mrf = new MetadataRelationsFetcherFromDatatable(ASyntaxProvider, this);

                if (ASyntaxProvider.IsSupportSchemas())
                {
                    mrf.PkSchemaFieldName = "TABLE_SCHEMA";
                    mrf.FkSchemaFieldName = "REFERENCED_TABLE_SCHEMA";
                }
                else
                {
                    mrf.PkDatabaseFieldName = "TABLE_SCHEMA";
                    mrf.FkDatabaseFieldName = "REFERENCED_TABLE_SCHEMA";
                }

                mrf.PkNameFieldName = "TABLE_NAME";
                mrf.PkFieldName = "COLUMN_NAME";
                mrf.FkNameFieldName = "REFERENCED_TABLE_NAME";
                mrf.FkFieldName = "REFERENCED_COLUMN_NAME";
                mrf.OrdinalFieldName = "ORDINAL_POSITION";
                mrf.Datatable = schemaTable;

                mrf.LoadMetadata(AMetadataFilter, AMetadataContainer, ADatabase);
            }
            catch (SQLiteException e)
            {
                throw (e);
            }
            catch (Exception e)
            {
                throw (e);
            }

        }
               
        
        public override void CreateAndBindInternalConnectionObj()
        {
            base.CreateAndBindInternalConnectionObj();
            Connection = new SQLiteConnection();
            AddInternalConnectionObject(Connection);
        }
        #endregion

        public StiSQLiteMetadataProvider()
            : this(null)
        {
        }

        public StiSQLiteMetadataProvider(IDbConnection connection) : base(connection)
        {
        }

        static StiSQLiteMetadataProvider()
        {
            Stimulsoft.Database.Helpers.MetadataProviderList.RegisterMetadataProvider(typeof(StiSQLiteMetadataProvider));
        }
    }
}
